/*
 * Copyright (C) 2008 Luke Kenneth Casson Leighton <lkcl@lkcl.net>
 * Copyright (C) 2008 Alp Toker <alp@atoker.com>
 * Copyright (C) 2008 Apple Inc.
 */
#ifndef __GdomCanvasRenderingContext2D_H__
#define __GdomCanvasRenderingContext2D_H__
#include <webkit/webkitdefines.h>
#include "gdom/GdomCanvasGradient.h"
#include "gdom/GdomDOMObject.h"
#include "gdom/GdomImageData.h"
#include "gdom/GdomTextMetrics.h"
#include "glib-object.h"


G_BEGIN_DECLS

typedef struct _GdomCanvasRenderingContext2D GdomCanvasRenderingContext2D;
typedef struct _GdomCanvasRenderingContext2DClass GdomCanvasRenderingContext2DClass;


#define GDOM_TYPE_CANVAS_RENDERING_CONTEXT2D            (gdom_canvas_rendering_context2d_get_type())
#define GDOM_CANVAS_RENDERING_CONTEXT2D(obj)            (G_TYPE_CHECK_INSTANCE_CAST((obj), GDOM_TYPE_CANVAS_RENDERING_CONTEXT2D, GdomCanvasRenderingContext2D))
#define GDOM_CANVAS_RENDERING_CONTEXT2D_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST((klass),  GDOM_TYPE_CANVAS_RENDERING_CONTEXT2D, GdomCanvasRenderingContext2D)Class)
#define GDOM_IS_CANVAS_RENDERING_CONTEXT2D(obj)         (G_TYPE_CHECK_INSTANCE_TYPE((obj), GDOM_TYPE_CANVAS_RENDERING_CONTEXT2D))
#define GDOM_IS_CANVAS_RENDERING_CONTEXT2D_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE((klass),  GDOM_TYPE_CANVAS_RENDERING_CONTEXT2D))
#define GDOM_CANVAS_RENDERING_CONTEXT2D_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS((obj),  GDOM_TYPE_CANVAS_RENDERING_CONTEXT2D, GdomCanvasRenderingContext2DClass))

typedef struct _GdomCanvasRenderingContext2DPrivate GdomCanvasRenderingContext2DPrivate;

struct _GdomCanvasRenderingContext2D {
    GdomDOMObject parent_instance;
    GdomCanvasRenderingContext2DPrivate *priv;
};

struct _GdomCanvasRenderingContext2DClass {
    GObjectClass parent_class;
};

WEBKIT_API GType
gdom_canvas_rendering_context2d_get_type (void);

WEBKIT_API void
gdom_canvas_rendering_context2d_save (GdomCanvasRenderingContext2D *thiz);

WEBKIT_API void
gdom_canvas_rendering_context2d_restore (GdomCanvasRenderingContext2D *thiz);

WEBKIT_API void
gdom_canvas_rendering_context2d_scale (GdomCanvasRenderingContext2D *thiz, gfloat sx, gfloat sy);

WEBKIT_API void
gdom_canvas_rendering_context2d_rotate (GdomCanvasRenderingContext2D *thiz, gfloat angle);

WEBKIT_API void
gdom_canvas_rendering_context2d_translate (GdomCanvasRenderingContext2D *thiz, gfloat tx, gfloat ty);

WEBKIT_API void
gdom_canvas_rendering_context2d_transform (GdomCanvasRenderingContext2D *thiz, gfloat m11, gfloat m12, gfloat m21, gfloat m22, gfloat dx, gfloat dy);

WEBKIT_API GdomCanvasGradient *
gdom_canvas_rendering_context2d_create_linear_gradient (GdomCanvasRenderingContext2D *thiz, gfloat x0, gfloat y0, gfloat x1, gfloat y1);

WEBKIT_API GdomCanvasGradient *
gdom_canvas_rendering_context2d_create_radial_gradient (GdomCanvasRenderingContext2D *thiz, gfloat x0, gfloat y0, gfloat r0, gfloat x1, gfloat y1, gfloat r1);

WEBKIT_API void
gdom_canvas_rendering_context2d_clear_rect (GdomCanvasRenderingContext2D *thiz, gfloat x, gfloat y, gfloat width, gfloat height);

WEBKIT_API void
gdom_canvas_rendering_context2d_fill_rect (GdomCanvasRenderingContext2D *thiz, gfloat x, gfloat y, gfloat width, gfloat height);

WEBKIT_API void
gdom_canvas_rendering_context2d_begin_path (GdomCanvasRenderingContext2D *thiz);

WEBKIT_API void
gdom_canvas_rendering_context2d_close_path (GdomCanvasRenderingContext2D *thiz);

WEBKIT_API void
gdom_canvas_rendering_context2d_move_to (GdomCanvasRenderingContext2D *thiz, gfloat x, gfloat y);

WEBKIT_API void
gdom_canvas_rendering_context2d_line_to (GdomCanvasRenderingContext2D *thiz, gfloat x, gfloat y);

WEBKIT_API void
gdom_canvas_rendering_context2d_quadratic_curve_to (GdomCanvasRenderingContext2D *thiz, gfloat cpx, gfloat cpy, gfloat x, gfloat y);

WEBKIT_API void
gdom_canvas_rendering_context2d_bezier_curve_to (GdomCanvasRenderingContext2D *thiz, gfloat cp1x, gfloat cp1y, gfloat cp2x, gfloat cp2y, gfloat x, gfloat y);

WEBKIT_API void
gdom_canvas_rendering_context2d_arc_to (GdomCanvasRenderingContext2D *thiz, gfloat x1, gfloat y1, gfloat x2, gfloat y2, gfloat radius);

WEBKIT_API void
gdom_canvas_rendering_context2d_rect (GdomCanvasRenderingContext2D *thiz, gfloat x, gfloat y, gfloat width, gfloat height);

WEBKIT_API void
gdom_canvas_rendering_context2d_arc (GdomCanvasRenderingContext2D *thiz, gfloat x, gfloat y, gfloat radius, gfloat start_angle, gfloat end_angle, gboolean anticlockwise);

WEBKIT_API void
gdom_canvas_rendering_context2d_fill (GdomCanvasRenderingContext2D *thiz);

WEBKIT_API void
gdom_canvas_rendering_context2d_stroke (GdomCanvasRenderingContext2D *thiz);

WEBKIT_API void
gdom_canvas_rendering_context2d_clip (GdomCanvasRenderingContext2D *thiz);

WEBKIT_API gboolean
gdom_canvas_rendering_context2d_is_point_in_path (GdomCanvasRenderingContext2D *thiz, gfloat x, gfloat y);


/* XXX TODO: custom function gdom_canvas_rendering_context2d_fill_text */


/* XXX TODO: custom function gdom_canvas_rendering_context2d_stroke_text */

WEBKIT_API GdomTextMetrics *
gdom_canvas_rendering_context2d_measure_text (GdomCanvasRenderingContext2D *thiz, gchar * text);

WEBKIT_API void
gdom_canvas_rendering_context2d_set_alpha (GdomCanvasRenderingContext2D *thiz, gfloat alpha);

WEBKIT_API void
gdom_canvas_rendering_context2d_set_composite_operation (GdomCanvasRenderingContext2D *thiz, gchar * composite_operation);

WEBKIT_API void
gdom_canvas_rendering_context2d_set_line_width (GdomCanvasRenderingContext2D *thiz, gfloat width);

WEBKIT_API void
gdom_canvas_rendering_context2d_set_line_cap (GdomCanvasRenderingContext2D *thiz, gchar * cap);

WEBKIT_API void
gdom_canvas_rendering_context2d_set_line_join (GdomCanvasRenderingContext2D *thiz, gchar * join);

WEBKIT_API void
gdom_canvas_rendering_context2d_set_miter_limit (GdomCanvasRenderingContext2D *thiz, gfloat limit);

WEBKIT_API void
gdom_canvas_rendering_context2d_clear_shadow (GdomCanvasRenderingContext2D *thiz);


/* XXX TODO: custom function gdom_canvas_rendering_context2d_set_stroke_color */


/* XXX TODO: custom function gdom_canvas_rendering_context2d_set_fill_color */


/* XXX TODO: custom function gdom_canvas_rendering_context2d_stroke_rect */


/* XXX TODO: custom function gdom_canvas_rendering_context2d_draw_image */


/* XXX TODO: custom function gdom_canvas_rendering_context2d_draw_image_from_rect */


/* XXX TODO: custom function gdom_canvas_rendering_context2d_set_shadow */


/* XXX TODO: custom function gdom_canvas_rendering_context2d_create_pattern */

WEBKIT_API GdomImageData *
gdom_canvas_rendering_context2d_create_image_data (GdomCanvasRenderingContext2D *thiz, gfloat sw, gfloat sh);

WEBKIT_API GdomImageData *
gdom_canvas_rendering_context2d_get_image_data (GdomCanvasRenderingContext2D *thiz, gfloat sx, gfloat sy, gfloat sw, gfloat sh);


/* XXX TODO: custom function gdom_canvas_rendering_context2d_put_image_data */

G_END_DECLS

#endif /* __GdomCanvasRenderingContext2D_H__ */
